package com.sun.j2ee.blueprints.petstore.util;

import com.sun.j2ee.blueprints.util.tracer.Debug;

import java.sql.*;

import com.sun.j2ee.blueprints.petstore.control.DuplicateAccountException;
import javax.servlet.http.HttpSession;
import javax.servlet.ServletContext;
import javax.ejb.SessionContext;

import org.jboss.security.SecurityAssociation;
import org.jboss.security.SimplePrincipal;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * This interface is implemented by a class that contains
 * non-portable security code.
 * @author Greg Murray
 */
public class JBossSecurityAdapter implements SecurityAdapter {

    /**
    * This method adds a user with the specified password to
    * the specified group within a realm.
    *
    * @throws  DuplicateAccountException if the userName
    *          already exists
    */

    public static final String SECURITY_CONTEXT = "SecurityContext";

    public void addUser(String realmName, String group, String userName, String password, SessionContext sc) throws DuplicateAccountException {

        Connection con = null;
        Statement stmt;

        ResultSet rs;

     // Table names should match what's configured in Tomcat JDBCRealm
     // (server.xml)

        String usersTable = "users";
        String rolesTable = "roles";
        String userRoleTable = "user_roles";

        javax.sql.DataSource dataSource = null;
        try {
            Context context = new InitialContext();
            dataSource = (javax.sql.DataSource)context.lookup("java:/SecurityDS");
        } catch ( NamingException e ) {
            System.err.print( "NamingException: " );
            System.err.println( e.getMessage() );
            return;
        }

        Debug.println("Checking "+ realmName +" Realm for userName=" +userName);
					
        try {

				con = dataSource.getConnection();
        		stmt = con.createStatement();

            String dupCheck = "SELECT user_name from " + usersTable + " WHERE user_name = '" + userName + "'"; 

            rs = stmt.executeQuery( dupCheck );

				if (rs.next()) {
                throw new DuplicateAccountException("The user account" + userName + " already exists");
				}	

	         String role = null;
  	         if (group.equals("gold")) {
            	role = "gold_customer";
        		}
        		else {
            	role = "customer";
        		}

        		Debug.println("Checking for valid role name:" + role);

            String roleCheck = "SELECT role_name from " + rolesTable + " WHERE role_name = '" + role + "'"; 
            rs = stmt.executeQuery( roleCheck );

				boolean validRole = false;

				while (rs.next()) {
					String r = rs.getString(1);
					if ( r.equals(role) )
						validRole = true;
				}

				if ( ! validRole )
            	throw new RuntimeException("Unable to add user: role " + role + " non-existent");

        Debug.println("Adding new User to " + realmName + " Realm userName=" + userName + " password=" + password + " group=" + group);

            String updateUsers = "INSERT INTO "+ usersTable + " VALUES(" + "'" + userName + "'," + "'" + password + "')"; 
            String updateUserRoles = "INSERT INTO "+userRoleTable +" VALUES(" + "'" + userName + "'," + "'" + role + "')";
            stmt.executeUpdate( updateUsers );
            stmt.executeUpdate( updateUserRoles );
            SecurityAssociation.setPrincipal( new SimplePrincipal( userName ) );
        } catch (SQLException ex) {
                System.err.println("SQLException: " + ex.getMessage());
        }

        finally {
            if (con != null) {
               try { con.close(); } catch (SQLException exception) { exception.printStackTrace(); }
            }
        }
    }

    /**
    * Programatically Logs a user into the form based login
    * mechamism of a web server when a new account is created.
    * This prevents the user from having to log in after
    * a new account is created.
    */
    public void loginUser(String userName, String password, ServletContext context, HttpSession session) {

        Debug.println("Logging in user " + userName + " to web server");

        session.setAttribute( "j_username", userName );
        session.setAttribute( "j_password", password );
    }
}
